Package: philr
Type: Package
Title: Phylogenetic partitioning based ILR transform for metagenomics
        data
Version: 1.37.0
Date: 2024-10-15
Authors@R: 
    c(person("Justin", "Silverman", role=c("aut", "cre"),
                     email = "jsilve24@gmail.com"),
      person(given = "Leo", family = "Lahti", role = c("ctb"),
             comment = c(ORCID = "0000-0001-5537-637X")))
Description: PhILR is short for Phylogenetic Isometric Log-Ratio Transform.
 This package provides functions for the analysis of compositional
 data (e.g., data representing proportions of different variables/parts).
 Specifically this package allows analysis of compositional data where the parts
 can be related through a phylogenetic tree  (as is common in microbiota survey
 data) and makes available the Isometric Log Ratio transform built from the
 phylogenetic tree and utilizing a weighted reference measure.
License: GPL-3
RoxygenNote: 7.3.2
Imports: ape, phangorn, tidyr, ggplot2, ggtree, methods
Suggests: testthat, knitr, ecodist, rmarkdown, BiocStyle, phyloseq,
        SummarizedExperiment, TreeSummarizedExperiment, glmnet, dplyr,
        mia
VignetteBuilder: knitr
biocViews: ImmunoOncology, Sequencing, Microbiome, Metagenomics,
        Software
URL: https://github.com/jsilve24/philr
BugReports: https://github.com/jsilve24/philr/issues
git_url: https://git.bioconductor.org/packages/philr
git_branch: devel
git_last_commit: 6f35184
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.23
Date/Publication: 2025-10-30
NeedsCompilation: no
Packaged: 2025-10-30 22:24:56 UTC; biocbuild
Author: Justin Silverman [aut, cre],
  Leo Lahti [ctb] (ORCID: <https://orcid.org/0000-0001-5537-637X>)
Maintainer: Justin Silverman <jsilve24@gmail.com>
